/*
 * GEXCEPT - Exception functions for glib
 * Copyright (C) 1999 Balazs Scheidler
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "gexcept.h"

GPrivate *except_block_stack = NULL;

GExceptBlock *g_except_block_new(void)
{
	GExceptBlock *ex_block = g_new0(GExceptBlock, 1);
	return ex_block;
}

/* pushes an exception block to the exception stack */
void g_except_block_push(GExceptBlock *ex_block)
{
	GSList *first_ex_block = g_private_get(except_block_stack);

	first_ex_block = g_slist_prepend(first_ex_block, ex_block);
	g_private_set(except_block_stack, first_ex_block);
}

/* indicates an error, if ex_block is not on the top of the exception stack */
void g_except_block_pop(GExceptBlock *ex_block)
{
	GSList *first_ex_block = g_private_get(except_block_stack);
	if (first_ex_block->data != ex_block) 
		g_error("g_except_block_pop(): popping a different exception block!!!\n");
	g_private_set(except_block_stack, g_slist_next(first_ex_block));
	g_free(first_ex_block->data);
	g_slist_free_1(first_ex_block);
}

/* creates an exception object */
GException *g_exception_new(gint category, const gchar *msg, gpointer params, const gchar *file, gint line)
{
	GException *ex = g_new(GException, 1);
	ex->category = category;
	ex->msg = msg;
	ex->params = params;
	ex->file = file;
	ex->line = line;
	return ex;
}

void g_exception_dump(GException *ex)
{
	g_message("Exception: (category=%d, msg=%s, params=%p, file=%s, line=%d)\n", ex->category, ex->msg, ex->params, ex->file, ex->line);
}

/* throws an exception */
void g_exception_throw(GException *ex)
{
	GSList *first_ex_block = g_private_get(except_block_stack);
	GExceptBlock *ex_block = first_ex_block->data;
	longjmp(ex_block->try_block, (int) ex);
}

void g_exceptions_init(void)
{
	GExceptBlock *def_ex_block;
	GException *ex;

	except_block_stack = g_private_new(NULL);
	def_ex_block = g_except_block_new();
	if ((ex = (GException *) setjmp(def_ex_block->try_block)))
		goto exception_not_handled;
	g_except_block_push(def_ex_block);
	return;

 exception_not_handled:
	if (ex) {
		g_error("Unhandled exception (category=%d, msg=%s, params=%p) at %s, line %d\n", ex->category, ex->msg, ex->params, ex->file, ex->line);
		exit(255);
	}
	else {
		g_error("Unhandled exception handler called with NULL exception object!!!\n");
		exit(255);
	}
}


