/*
 * GEXCEPT - Exception functions for glib
 * Copyright (C) 1999 Balazs Scheidler
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __G_EXCEPT_H__
#define __G_EXCEPT_H__

#include <glib.h>
#include <setjmp.h>

typedef struct _GExceptBlock GExceptBlock;
typedef struct _GException GException;

struct _GExceptBlock {
	jmp_buf    try_block;
};

struct _GException {
	gint category;
	const gchar *msg;
	void *params;
	const gchar *file;
	gint line;
};

/* FIXME: these macros pollute namespace */

#define G_TRY \
  do { \
    GExceptBlock *__ex_block = g_except_block_new(); \
    GException *__ex = NULL; \
    if ((__ex = (GException *) setjmp(__ex_block->try_block))) \
      goto __catch_block; \
    g_except_block_push(__ex_block);

#define G_CATCH_START(ex) \
    g_except_block_pop(__ex_block); \
    break; \
__catch_block: \
    { \
      GException *ex = __ex; \
      g_except_block_pop(__ex_block); \
      if (!__ex) { \
	      g_error("G_CATCH_START macro, NULL exception received!!!\n"); \
              break; \
      } \
      switch (__ex->category)

#define G_CATCH(cat) \
      case cat:

#define G_CATCH_END \
    } \
  } while(0);

#define G_THROW(cat, msg, param) G_RETHROW(g_exception_new(cat, msg, param, __FILE__, __LINE__))
#define G_RETHROW(ex) g_exception_throw(ex)

GExceptBlock *g_except_block_new(void);

/* pushes an exception block to the exception stack */
void g_except_block_push(GExceptBlock *ex_block);

/* indicates an error, if ex_block is not on the top of the exception stack */
void g_except_block_pop(GExceptBlock *ex_block);

/* creates an exception object */
GException *g_exception_new(gint category, const gchar *msg, gpointer params, const gchar *file, gint line);

/* prints all information about an exception using g_message */
void g_exception_dump(GException *ex);

/* throws an exception */
void g_exception_throw(GException *ex);

/* initializes exception subsystem */
void g_exceptions_init(void);
    
#endif
